@@ -40,24 +40,30 @@ class Field(JsonSerializable):
     def __get__(self, instance, owner):
         if instance is None:
             raise Exception('Cannot access field without instance')
-        return instance._data.get(self.serialized_name, None)
+        return instance._data.get(self.serialized_name, self.default_value)
 
     # noinspection PyProtectedMember
     def __set__(self, instance, value, test: bool = False):
         if not self.null:
-            assert value is not None, "Null value passed for not nullable field \'" + self.name + "\' in class " + str(type(instance))
+            assert value is not None, "Null value passed for not nullable field \'" + self.name + "\' in class " + str(
+                type(instance))
         if test is False:
             if instance is None:
                 raise Exception('Cannot access field without instance')
             instance._data[self.serialized_name] = value
 
-    def __init__(self, field_type: FieldType, serialized_name: Optional[str] = None, null: bool = True):
+    def __init__(self, field_type: FieldType,
+                 serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[Any] = None):
         self.field_type = field_type
         self.serialized_name = serialized_name  # field name in serialized json
         self.name = None  # field name in the defined class
         self.null = null
+        self.default_value = default_value
 
 
+# noinspection DuplicatedCode,PyBroadException
 class IntegerField(Field):
     def __set__(self, instance, value, test: bool = False):
         if value is not None and isinstance(value, str) and value != '' and self.accepts_string:
@@ -73,10 +79,16 @@ class IntegerField(Field):
             assert value <= self.max_value
         super().__set__(instance, value, test)
 
-    def __init__(self, serialized_name: Optional[str] = None, null: bool = True, accepts_string: bool = False,
+    def __init__(self, serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[int] = None,
+                 accepts_string: bool = False,
                  min_value: Optional[int] = None, max_value: Optional[int] = None):
-        super(IntegerField, self).__init__(field_type=FieldType.INTEGER, serialized_name=serialized_name, null=null)
+        super(IntegerField, self).__init__(field_type=FieldType.INTEGER,
+                                           serialized_name=serialized_name,
+                                           null=null,
+                                           default_value=default_value)
         self.accepts_string = accepts_string
+        assert default_value is None or isinstance(default_value, int)
         assert min_value is None or isinstance(min_value, int)
         assert max_value is None or isinstance(max_value, int)
         if min_value is not None and max_value is not None:
@@ -85,6 +97,7 @@ class IntegerField(Field):
         self.max_value = max_value
 
 
+# noinspection DuplicatedCode
 class FloatField(Field):
     def __set__(self, instance, value, test: bool = False):
         if value is not None and isinstance(value, str) and value != '' and self.accepts_string:
@@ -102,10 +115,16 @@ class FloatField(Field):
             assert value <= self.max_value
         super().__set__(instance, value, test)
 
-    def __init__(self, serialized_name: Optional[str] = None, null: bool = True, accepts_string: bool = False,
+    def __init__(self, serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[float] = None,
+                 accepts_string: bool = False,
                  min_value: Optional[float] = None, max_value: Optional[float] = None):
-        super(FloatField, self).__init__(field_type=FieldType.FLOAT, serialized_name=serialized_name, null=null)
+        super(FloatField, self).__init__(field_type=FieldType.FLOAT,
+                                         serialized_name=serialized_name,
+                                         null=null,
+                                         default_value=default_value)
         self.accepts_string = accepts_string
+        assert default_value is None or isinstance(default_value, float)
         assert min_value is None or isinstance(min_value, float)
         assert max_value is None or isinstance(max_value, float)
         if min_value is not None and max_value is not None:
@@ -116,7 +135,8 @@ class FloatField(Field):
 
 class BooleanField(Field):
     def __set__(self, instance, value, test: bool = False):
-        if value is not None and isinstance(value, str) and value in ['True', 'False', 'true', 'false'] and self.accepts_string:
+        if value is not None and isinstance(value, str) and value in ['True', 'False', 'true',
+                                                                      'false'] and self.accepts_string:
             if value in ['True', 'true']:
                 value = True
             elif value in ['False', 'false']:
@@ -125,9 +145,17 @@ class BooleanField(Field):
             raise TypeError(instance, self.name, bool, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, serialized_name: Optional[str] = None, null: bool = True, accepts_string: bool = False):
-        super(BooleanField, self).__init__(field_type=FieldType.BOOLEAN, serialized_name=serialized_name, null=null)
+    def __init__(self, serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[float] = None,
+                 accepts_string: bool = False,
+                 ):
+        super(BooleanField, self).__init__(field_type=FieldType.BOOLEAN,
+                                           serialized_name=serialized_name,
+                                           null=null,
+                                           default_value=default_value)
         self.accepts_string = accepts_string
+        assert default_value is None or isinstance(default_value, bool)
 
 
 class StringField(Field):
@@ -138,9 +166,17 @@ class StringField(Field):
             raise TypeError(instance, self.name, str, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, serialized_name: Optional[str] = None, null: bool = True, accepts_non_string: bool = False):
-        super(StringField, self).__init__(field_type=FieldType.STRING, serialized_name=serialized_name, null=null)
+    def __init__(self, serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[str] = None,
+                 accepts_non_string: bool = False,
+                 ):
+        super(StringField, self).__init__(field_type=FieldType.STRING,
+                                          serialized_name=serialized_name,
+                                          null=null,
+                                          default_value=default_value)
         self.accepts_non_string = accepts_non_string
+        assert default_value is None or isinstance(default_value, str)
 
 
 class BytesField(Field):
@@ -149,8 +185,14 @@ class BytesField(Field):
             raise TypeError(instance, self.name, bytes, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, serialized_name: Optional[str] = None, null: bool = True):
-        super(BytesField, self).__init__(field_type=FieldType.BYTES, serialized_name=serialized_name, null=null)
+    def __init__(self, serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[bytes] = None):
+        super(BytesField, self).__init__(field_type=FieldType.BYTES,
+                                         serialized_name=serialized_name,
+                                         null=null,
+                                         default_value=default_value
+                                         )
+        assert default_value is None or isinstance(default_value, bytes)
 
 
 class ByteArrayField(Field):
@@ -159,10 +201,16 @@ class ByteArrayField(Field):
             raise TypeError(instance, self.name, bytearray, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, serialized_name: Optional[str] = None, null: bool = True):
-        super(ByteArrayField, self).__init__(field_type=FieldType.BYTE_ARRAY, serialized_name=serialized_name, null=null)
+    def __init__(self, serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[bytearray] = None):
+        super(ByteArrayField, self).__init__(field_type=FieldType.BYTE_ARRAY,
+                                             serialized_name=serialized_name,
+                                             null=null,
+                                             default_value=default_value)
+        assert default_value is None or isinstance(default_value, bytearray)
 
 
+# noinspection DuplicatedCode
 class MultipleChoiceStringField(Field):
     def __set__(self, instance, value, test: bool = False):
         if value is not None and not isinstance(value, str):
@@ -171,20 +219,31 @@ class MultipleChoiceStringField(Field):
             raise ValueError('Invalid value ' + str(value) + ' not present in options ' + str(self.options))
         super().__set__(instance, value, test)
 
-    def __init__(self, options: Union[List[str], Set[str]], serialized_name: Optional[str] = None, null: bool = True):
-        super(MultipleChoiceStringField, self).__init__(field_type=FieldType.STRING, serialized_name=serialized_name, null=null)
+    def __init__(self, options: Union[List[str], Set[str]],
+                 serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[str] = None
+                 ):
+        super(MultipleChoiceStringField, self).__init__(field_type=FieldType.STRING,
+                                                        serialized_name=serialized_name,
+                                                        default_value=default_value,
+                                                        null=null)
         if options is None:
             raise Exception("Null options passed for multiple choice string field")
         if not (isinstance(options, list) or isinstance(options, set)):
-            raise Exception("Invalid type for options passed for multiple choice string field, must be either set or list but found " + str(type(options)))
+            raise Exception(
+                "Invalid type for options passed for multiple choice string field, must be either set or list but "
+                "found " + str(type(options)))
         if len(options) == 0:
             raise Exception("Empty options passed for enum string field")
         for option in options:
             if not isinstance(option, str):
-                raise Exception("Invalid value in options of multiple choice string field, " + str(option) + ', expected str value but found ' + str(type(option)))
+                raise Exception("Invalid value in options of multiple choice string field, " + str(
+                    option) + ', expected str value but found ' + str(type(option)))
         if len(options) != len(set(options)):
             raise Exception("Duplicate values passed for options of multiple choice string field")
         self.options = set(options)
+        assert default_value is None or (isinstance(default_value, str) and default_value in self.options)
 
 
 class EnumStringField(Field):
@@ -197,8 +256,12 @@ class EnumStringField(Field):
             raise ValueError('Invalid value ' + str(value) + ' not present in enum values ' + str(self.options))
         super().__set__(instance, value, test)
 
-    def __init__(self, enum, serialized_name: Optional[str] = None, null: bool = True):
-        super(EnumStringField, self).__init__(field_type=FieldType.STRING, serialized_name=serialized_name, null=null)
+    def __init__(self, enum, serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[str] = None):
+        super(EnumStringField, self).__init__(field_type=FieldType.STRING,
+                                              serialized_name=serialized_name,
+                                              null=null,
+                                              default_value=default_value)
         if enum is None:
             raise Exception("Null enum passed for enum string field")
         if not issubclass(enum, Enum):
@@ -208,13 +271,17 @@ class EnumStringField(Field):
             raise Exception("Enum with no values for enum string field")
         for option in options:
             if not isinstance(option, str):
-                raise Exception("Invalid value in enum string field, " + str(option) + ', expected str value but found ' + str(type(option)))
+                raise Exception(
+                    "Invalid value in enum string field, " + str(option) + ', expected str value but found ' + str(
+                        type(option)))
         if len(options) != len(set(options)):
             raise Exception("Duplicate values passed for options of enum string field")
         self.enum_options = [e for e in enum]
         self.options = set(options)
+        assert default_value is None or (isinstance(default_value, str) and default_value in self.options)
 
 
+# noinspection DuplicatedCode
 class MultipleChoiceIntegerField(Field):
     def __set__(self, instance, value, test: bool = False):
         if value is not None and not isinstance(value, int):
@@ -223,20 +290,28 @@ class MultipleChoiceIntegerField(Field):
             raise ValueError('Invalid value ' + str(value) + ' not present in options ' + str(self.options))
         super().__set__(instance, value, test)
 
-    def __init__(self, options: Union[List[int], Set[int]], serialized_name: Optional[str] = None, null: bool = True):
-        super(MultipleChoiceIntegerField, self).__init__(field_type=FieldType.INTEGER, serialized_name=serialized_name, null=null)
+    def __init__(self, options: Union[List[int], Set[int]], serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[int] = None):
+        super(MultipleChoiceIntegerField, self).__init__(field_type=FieldType.INTEGER,
+                                                         serialized_name=serialized_name,
+                                                         null=null,
+                                                         default_value=default_value)
         if options is None:
             raise Exception("Null options passed for multiple choice integer field")
         if not (isinstance(options, list) or isinstance(options, set)):
-            raise Exception("Invalid type for options passed for multiple choice integer field, must be either set or list but found " + str(type(options)))
+            raise Exception(
+                "Invalid type for options passed for multiple choice integer field, must be either set or list but "
+                "found " + str(type(options)))
         if len(options) == 0:
             raise Exception("Empty options passed for multiple choice integer field")
         for option in options:
             if not isinstance(option, int):
-                raise Exception("Invalid value in options of multiple choice integer field, " + str(option) + ', expected int value but found ' + str(type(option)))
+                raise Exception("Invalid value in options of multiple choice integer field, " + str(
+                    option) + ', expected int value but found ' + str(type(option)))
         if len(options) != len(set(options)):
             raise Exception("Duplicate values passed for options of multiple choice integer field")
         self.options = set(options)
+        assert default_value is None or (isinstance(default_value, int) and default_value in self.options)
 
 
 class EnumIntegerField(Field):
@@ -249,8 +324,12 @@ class EnumIntegerField(Field):
             raise ValueError('Invalid value ' + str(value) + ' not present in enum values ' + str(self.options))
         super().__set__(instance, value, test)
 
-    def __init__(self, enum, serialized_name: Optional[str] = None, null: bool = True):
-        super(EnumIntegerField, self).__init__(field_type=FieldType.INTEGER, serialized_name=serialized_name, null=null)
+    def __init__(self, enum, serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[int] = None):
+        super(EnumIntegerField, self).__init__(field_type=FieldType.INTEGER,
+                                               serialized_name=serialized_name,
+                                               null=null,
+                                               default_value=default_value)
         if enum is None:
             raise Exception("Null enum passed for enum string field")
         if not issubclass(enum, Enum):
@@ -260,15 +339,20 @@ class EnumIntegerField(Field):
             raise Exception("Enum with no values passed for enum integer field")
         for option in options:
             if not isinstance(option, int):
-                raise Exception("Invalid value in enum integer field, " + str(option) + ', expected int value but found ' + str(type(option)))
+                raise Exception(
+                    "Invalid value in enum integer field, " + str(option) + ', expected int value but found ' + str(
+                        type(option)))
         if len(options) != len(set(options)):
             raise Exception("Duplicate values passed for options of enum integer field")
         self.enum_options = [e for e in enum]
         self.options = set(options)
+        assert default_value is None or (isinstance(default_value, int) and default_value in self.options)
 
 
 class DateField(Field):
     def get_json_formatted_value(self, value):
+        if value is None:
+            return None
         return datetime.date.strftime(value, self.date_format)
 
     def __set__(self, instance, value, test: bool = False):
@@ -281,13 +365,20 @@ class DateField(Field):
             raise TypeError(instance, self.name, datetime.date, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, date_format: str = '%Y-%m-%d', serialized_name: Optional[str] = None, null: bool = True):
-        super(DateField, self).__init__(field_type=FieldType.DATE, serialized_name=serialized_name, null=null)
+    def __init__(self, date_format: str = '%Y-%m-%d', serialized_name: Optional[str] = None,
+                 null: bool = True, default_value: Optional[datetime.date] = None):
+        super(DateField, self).__init__(field_type=FieldType.DATE,
+                                        serialized_name=serialized_name,
+                                        null=null,
+                                        default_value=default_value)
         self.date_format = date_format
+        assert default_value is None or isinstance(default_value, datetime.date)
 
 
 class TimeField(Field):
     def get_json_formatted_value(self, value):
+        if value is None:
+            return None
         return datetime.time.strftime(value, self.time_format)
 
     def __set__(self, instance, value, test: bool = False):
@@ -300,9 +391,14 @@ class TimeField(Field):
             raise TypeError(instance, self.name, datetime.time, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, time_format: str = '%H:%M:%S', serialized_name: Optional[str] = None, null: bool = True):
-        super(TimeField, self).__init__(field_type=FieldType.TIME, serialized_name=serialized_name, null=null)
+    def __init__(self, time_format: str = '%H:%M:%S', serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[datetime.time] = None):
+        super(TimeField, self).__init__(field_type=FieldType.TIME,
+                                        serialized_name=serialized_name,
+                                        null=null,
+                                        default_value=default_value)
         self.time_format = time_format
+        assert default_value is None or isinstance(default_value, datetime.time)
 
 
 class DateTimeField(Field):
@@ -317,56 +413,84 @@ class DateTimeField(Field):
                 try:
                     dt = datetime.datetime.strptime(value, self.datetime_format)
                 except ValueError:
-                    raise Exception('Error parsing date ' + str(value) + ' with given format ' + str(self.datetime_format))
+                    raise Exception(
+                        'Error parsing date ' + str(value) + ' with given format ' + str(self.datetime_format))
             else:
                 dt = parser.parse(value)
-            value = pytz.timezone(self.datetime_timezone).localize(dt) if dt.tzinfo is None or dt.tzinfo.utcoffset(dt) is None else dt
+            value = pytz.timezone(self.datetime_timezone).localize(dt) if dt.tzinfo is None or dt.tzinfo.utcoffset(
+                dt) is None else dt
         if value is not None and not isinstance(value, datetime.datetime):
             raise TypeError(instance, self.name, datetime.datetime, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, datetime_format: Optional[str] = '%Y-%m-%d %H:%M:%S', datetime_timezone: str = 'UTC', serialized_name: Optional[str] = None, null: bool = True):
-        super(DateTimeField, self).__init__(field_type=FieldType.DATETIME, serialized_name=serialized_name, null=null)
+    def __init__(self, datetime_format: Optional[str] = '%Y-%m-%d %H:%M:%S',
+                 datetime_timezone: str = 'UTC',
+                 serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[datetime.datetime] = None):
+        super(DateTimeField, self).__init__(field_type=FieldType.DATETIME,
+                                            serialized_name=serialized_name,
+                                            null=null,
+                                            default_value=default_value)
         self.datetime_format = datetime_format
         self.datetime_timezone = datetime_timezone
+        assert default_value is None or isinstance(default_value, datetime.datetime)
 
 
 class TimestampSecondsField(Field):
     def get_json_formatted_value(self, value):
+        if value is None:
+            return None
         return int(value.replace(tzinfo=pytz.timezone(self.datetime_timezone)).timestamp())
 
     def __set__(self, instance, value, test: bool = False):
         if value is not None and isinstance(value, int):
             try:
                 value = pytz.timezone(self.datetime_timezone).localize(datetime.datetime.fromtimestamp(float(value)))
-            except:
+            except Exception:
                 raise Exception('Error parsing timestamp (in seconds) ' + str(value))
         if value is not None and not isinstance(value, datetime.datetime):
             raise TypeError(instance, self.name, datetime.datetime, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, datetime_timezone: str = 'UTC', serialized_name: Optional[str] = None, null: bool = True):
-        super(TimestampSecondsField, self).__init__(field_type=FieldType.DATETIME, serialized_name=serialized_name, null=null)
+    def __init__(self, datetime_timezone: str = 'UTC', serialized_name: Optional[str] = None, null: bool = True,
+                 default_value: Optional[datetime.datetime] = None):
+        super(TimestampSecondsField, self).__init__(field_type=FieldType.DATETIME,
+                                                    serialized_name=serialized_name,
+                                                    null=null,
+                                                    default_value=default_value)
         self.datetime_timezone = datetime_timezone
+        assert default_value is None or isinstance(default_value, datetime.datetime)
 
 
 class TimestampMillisecondsField(Field):
     def get_json_formatted_value(self, value):
+        if value is None:
+            return None
         return int(value.replace(tzinfo=pytz.timezone(self.datetime_timezone)).timestamp() * 1000.0)
 
     def __set__(self, instance, value, test: bool = False):
         if value is not None and isinstance(value, int):
             try:
-                value = pytz.timezone(self.datetime_timezone).localize(datetime.datetime.fromtimestamp(float(value / 1000.0)))
-            except:
+                value = pytz.timezone(self.datetime_timezone).localize(
+                    datetime.datetime.fromtimestamp(float(value / 1000.0)))
+            except Exception:
                 raise Exception('Error parsing timestamp (in milliseconds) ' + str(value))
         if value is not None and not isinstance(value, datetime.datetime):
             raise TypeError(instance, self.name, datetime.datetime, value)
         super().__set__(instance, value, test)
 
-    def __init__(self, datetime_timezone: str = 'UTC', serialized_name: Optional[str] = None, null: bool = True):
-        super(TimestampMillisecondsField, self).__init__(field_type=FieldType.DATETIME, serialized_name=serialized_name, null=null)
+    def __init__(self,
+                 datetime_timezone: str = 'UTC',
+                 serialized_name: Optional[str] = None,
+                 null: bool = True,
+                 default_value: Optional[datetime.datetime] = None):
+        super(TimestampMillisecondsField, self).__init__(field_type=FieldType.DATETIME,
+                                                         serialized_name=serialized_name,
+                                                         null=null,
+                                                         default_value=default_value)
         self.datetime_timezone = datetime_timezone
+        assert default_value is None or isinstance(default_value, datetime.datetime)
 
 
 class JsonField(Field):
@@ -382,6 +506,7 @@ class JsonField(Field):
 F = TypeVar('F', bound=Field)
 
 
+# noinspection PyProtectedMember
 class ListField(Field):
     def __set__(self, instance, value, test: bool = False):
         if value is not None and not isinstance(value, list):
@@ -409,7 +534,8 @@ class ListField(Field):
         super(ListField, self).__init__(field_type=FieldType.LIST, serialized_name=serialized_name, null=null)
         valid_types = [int, str, bool, float]
         if not isinstance(item_type, Field):
-            assert item_type in valid_types, 'Invalid list item type ' + str(item_type) + 'must be in ' + str(valid_types)
+            assert item_type in valid_types, 'Invalid list item type ' + str(item_type) + 'must be in ' + str(
+                valid_types)
         else:
             assert item_type.serialized_name is None, 'List item type should not have serialized name'
         self.item_type = item_type
@@ -419,20 +545,21 @@ class JsonObjectMeta(type):
     @staticmethod
     def __get_class_hierarchy_field_names(cls) -> List[str]:
         tmp_class_dict = cls.__dict__
-        model_field_names = [k for k in tmp_class_dict.keys() if (isinstance(tmp_class_dict.get(k), JsonSerializable))]  # type: List[str]
+        model_field_names = [k for k in tmp_class_dict.keys() if
+                             (isinstance(tmp_class_dict.get(k), JsonSerializable))]  # type: List[str]
         for base in cls.__bases__:
             if issubclass(base, JsonSerializable):
                 model_field_names.extend(JsonObjectMeta.__get_class_hierarchy_field_names(base))
         return model_field_names
 
-    def __new__(cls, name, bases, attrs: Dict[str, Any]):
+    def __new__(mcs, name, bases, attrs: Dict[str, Any]):
         m_module = attrs.pop('__module__')
         new_attrs = {'__module__': m_module}
         class_cell = attrs.pop('__classcell__', None)
         if class_cell is not None:
             new_attrs['__classcell__'] = class_cell
 
-        new_class = super(JsonObjectMeta, cls).__new__(cls, name, bases, new_attrs)
+        new_class = super(JsonObjectMeta, mcs).__new__(mcs, name, bases, new_attrs)
 
         # attr_meta = attrs.pop('Meta', None)
         # if not attr_meta:
@@ -447,13 +574,16 @@ class JsonObjectMeta(type):
                     if field.serialized_name is None:
                         field.serialized_name = field_name
                     if field.serialized_name in serialized_names:
-                        raise Exception('Duplicate serialized names \'' + str(field.serialized_name) + '\' found in ' + str(name) + ' class')
+                        raise Exception(
+                            'Duplicate serialized names \'' + str(field.serialized_name) + '\' found in ' + str(
+                                name) + ' class')
                     serialized_names.append(field.serialized_name)
                     field.name = field_name
                     setattr(new_class, field.name, field)
                 else:
                     if field_name in serialized_names:
-                        raise Exception('Duplicate serialized names \'' + str(field_name) + '\' found in ' + str(name) + ' class')
+                        raise Exception(
+                            'Duplicate serialized names \'' + str(field_name) + '\' found in ' + str(name) + ' class')
                     serialized_names.append(field_name)
                     field.serialized_name = field_name
                     setattr(new_class, field_name, field)
@@ -463,7 +593,8 @@ class JsonObjectMeta(type):
         user_defined_init = new_class.__init__
 
         # noinspection PyUnusedLocal
-        def my_custom_init(instance_self, accept_unknown: bool = False, extra_attributes: Optional[List[str]] = None, *init_args, **init_kwargs):
+        def my_custom_init(instance_self, accept_unknown: bool = False, extra_attributes: Optional[List[str]] = None,
+                           *init_args, **init_kwargs):
             instance_self._data = {}  # dict.fromkeys(attrs.keys())
             _setattr = setattr
 
@@ -485,7 +616,8 @@ class JsonObjectMeta(type):
                 elif extra_attributes is not None and key in extra_attributes:
                     _setattr(instance_self, key, value)
                 elif not accept_unknown:
-                    raise Exception("value given in instance initialization but was not defined in model class (" + str(type(instance_self)) + ")as Field. key:" + str(key) +
+                    raise Exception("value given in instance initialization but was not defined in model class (" + str(
+                        type(instance_self)) + ")as Field. key:" + str(key) +
                                     " val:" + str(value) + " type(value):" + str(type(value)))
 
                 # if user_defined_init != JsonObject.__init__:
@@ -529,7 +661,8 @@ class ObjectListField(Field, List[T], Generic[T]):
             value = []
         for item in value:
             assert item is not None, "Null item passed to ObjectListField"
-            assert isinstance(item, self.item_type), "ObjectListField items must be of " + str(self.item_type) + ", found " + str(type(item))
+            assert isinstance(item, self.item_type), "ObjectListField items must be of " + str(
+                self.item_type) + ", found " + str(type(item))
         super(ObjectListField, self).__set__(instance, value, test)
 
     def __init__(self, item_type: Type[T], serialized_name: Optional[str] = None, null: bool = True):
@@ -541,12 +674,15 @@ class TypeHierarchyAdapter:
     def __init__(self,
                  base_class: Type[T],
                  type_key: str,
-                 subtype_key_values: Dict[str, Type[T]]):
+                 subtype_key_values: Dict[str, Type[T]],
+                 accept_sub_type: bool = False):
         self.base_class = base_class
         self.type_key = type_key
+        self.accept_sub_type = accept_sub_type
         self.subtype_key_values = subtype_key_values
 
 
+# noinspection DuplicatedCode
 class Pykson:
     @staticmethod
     def __get_fields_mapped_by_names(cls) -> Dict[str, Field]:
@@ -689,13 +825,16 @@ class Pykson:
 
         for type_hierarchy_adapter in self.type_hierarchy_adapters:
             # if type_hierarchy_adapter.base_class == cls:
-            if issubclass(cls, type_hierarchy_adapter.base_class):
+            if (type_hierarchy_adapter.accept_sub_type is True and issubclass(cls, type_hierarchy_adapter.base_class)) \
+                    or type_hierarchy_adapter.base_class == cls:
                 subtype_key = data.get(type_hierarchy_adapter.type_key, None)
                 if subtype_key is None:
-                    raise Exception('No sub-type key provided in class of type ' + str(cls) + ' for type key ' + str(type_hierarchy_adapter.type_key))
+                    raise Exception('No sub-type key provided in class of type ' + str(cls) + ' for type key ' + str(
+                        type_hierarchy_adapter.type_key))
                 sub_type = type_hierarchy_adapter.subtype_key_values.get(subtype_key, None)
                 if sub_type is None:
-                    raise Exception('No sub-type provided in type hierarchy adapter for base class of ' + str(cls) + ' for sub-type key ' + str(subtype_key))
+                    raise Exception('No sub-type provided in type hierarchy adapter for base class of ' + str(
+                        cls) + ' for sub-type key ' + str(subtype_key))
                 extra_attributes.append(type_hierarchy_adapter.type_key)
 
         children_mapped_by_serialized_names = Pykson.__get_children_mapped_by_serialized_names(sub_type)
@@ -709,7 +848,8 @@ class Pykson:
                 for data_value_item in data_value:
                     # noinspection PyUnresolvedReferences
                     data_list_value.append(
-                        self.from_json(data_value_item, fields_mapped_by_serialized_names[data_key].item_type, accept_unknown=accept_unknown)
+                        self.from_json(data_value_item, fields_mapped_by_serialized_names[data_key].item_type,
+                                       accept_unknown=accept_unknown)
                     )
                 data_copy[field_names_mapped_by_serialized_names[data_key]] = data_list_value
             # elif isinstance(data_value, list) and (data_key in fields_mapped_by_serialized_names.keys()) and \
@@ -721,11 +861,15 @@ class Pykson:
             #         else:
             #             data_copy[data_key] = data_value
             elif data_key in children_mapped_by_serialized_names.keys() and isinstance(data_value, dict):
-                data_copy[data_key] = self.from_json(data_value, type(children_mapped_by_serialized_names[data_key]), accept_unknown=accept_unknown)
-            elif data_key in fields_mapped_by_serialized_names.keys() and isinstance(fields_mapped_by_serialized_names[data_key], ObjectField):
+                data_copy[data_key] = self.from_json(data_value, type(children_mapped_by_serialized_names[data_key]),
+                                                     accept_unknown=accept_unknown)
+            elif data_key in fields_mapped_by_serialized_names.keys() and isinstance(
+                    fields_mapped_by_serialized_names[data_key], ObjectField):
                 # noinspection PyUnresolvedReferences
-                data_copy[field_names_mapped_by_serialized_names[data_key]] = self.from_json(data_value, fields_mapped_by_serialized_names[data_key].item_type,
-                                                                                             accept_unknown=accept_unknown)
+                data_copy[field_names_mapped_by_serialized_names[data_key]] = \
+                    self.from_json(data_value,
+                                   fields_mapped_by_serialized_names[data_key].item_type,
+                                   accept_unknown=accept_unknown)
             else:
                 if data_key in field_names_mapped_by_serialized_names.keys():
                     data_copy[field_names_mapped_by_serialized_names[data_key]] = data_value
@@ -741,7 +885,8 @@ class Pykson:
             list_result.append(self.from_json(data_value_item, cls, accept_unknown))
         return list_result
 
-    def from_csv(self, data: str, cls: Type[T], line_separator: str = '\n', first_row_as_field_names: bool = True, accept_unknown: bool = False) -> List[T]:
+    def from_csv(self, data: str, cls: Type[T], line_separator: str = '\n', first_row_as_field_names: bool = True,
+                 accept_unknown: bool = False) -> List[T]:
         data_items = data.split(line_separator)
         if first_row_as_field_names:
             reader_list = csv.DictReader(data_items)
@@ -751,7 +896,8 @@ class Pykson:
         rows = [r for r in reader_list]
         return self._from_json_list(rows, cls=cls, accept_unknown=accept_unknown)
 
-    def from_json(self, data: Union[str, Dict, List], cls: Type[T], accept_unknown: bool = False) -> Optional[Union[T, List[T]]]:
+    def from_json(self, data: Union[str, Dict, List], cls: Type[T], accept_unknown: bool = False
+                  ) -> Optional[Union[T, List[T]]]:
         if isinstance(data, str):
             data = json.loads(data)
         if isinstance(data, dict):
@@ -803,7 +949,8 @@ class Pykson:
                             final_dict[type_hierarchy_adapter.type_key] = subtype_key
                             break
                     if not type_found:
-                        raise Exception('No sub-type key was entered for item of type ' + str(type(item)) + ' in type hierarchy of base type ' +
+                        raise Exception('No sub-type key was entered for item of type ' + str(
+                            type(item)) + ' in type hierarchy of base type ' +
                                         str(type_hierarchy_adapter.base_class))
 
             for field_key, field_value in fields_dict.items():
